@@ -480,83 +480,101 @@ checkNumberMargins = function(data, marginMatrix) {
 #' @param calibrationVariable name of the calibration varaible for which regroupment has to be done
 #' @param vecModalities Initial modalities of the variable
 #' @param newModality Regrouped modalities of the variable
+#' 
+#' @examples
+#' ## Suppose we have a calibration matrix and a margin matrix containing information
+#' ## for two categorical variables "X1" (10 modalities) and "X2" (5 modalities)
+#' 
+#' matrixCal <- data.frame(matrix(
+#'                c(floor(10*runif(100))+1,floor((5)*runif(100))+1,
+#'                floor(10*runif(100))+1,rep(10,100)),
+#'                ncol=4))
+#' marginMatrix <- matrix(c("X1",10,rep(1/10,10),
+#'                  "X2",5,rep(1/5,5),rep(0,5)), nrow=2, byrow=TRUE)
+#' 
+#' # table(matrixCal$X1)
+#' # 1  2  3  4  5  6  7  8  9 10 
+#' # 9  8  8  8 11 15 13  6 10 12 
+#' # marginMatrix
+#' # [,1] [,2] [,3]  [,4]  [,5]  [,6]  [,7]  [,8]  [,9]  [,10] [,11] [,12]
+#' # [1,] "X1" "10" "0.1" "0.1" "0.1" "0.1" "0.1" "0.1" "0.1" "0.1" "0.1" "0.1"
+#' # [2,] "X2" "5"  "0.2" "0.2" "0.2" "0.2" "0.2" "0"   "0"   "0"   "0"   "0" 
+#' 
+#' regroupCalibrationModalities(matrixCal, marginMatrix, "X1", c(3,4,8), "0")
+#' 
+#' # table(matrixCal$X1)
+#' # 0  1  2  5  6  7  9 10 
+#' # 22  9  8 11 15 13 10 12 
+#' # marginMatrix
+#' # [,1] [,2] [,3]  [,4]  [,5]  [,6]  [,7]  [,8]  [,9]  [,10]
+#' # [1,] "X1" "8"  "0.3" "0.1" "0.1" "0.1" "0.1" "0.1" "0.1" "0.1"
+#' # [2,] "X2" "5"  "0.2" "0.2" "0.2" "0.2" "0.2" "0"   "0"   "0"  
+#' 
 #' @export
 regroupCalibrationModalities <- function(calibrationMatrix, marginMatrix, calibrationVariable, vecModalities, newModality) {
-
+  
   # First, check if number of modalities match in calibrationMatrix and marginMatrix,
   # otherwise stop
   if(!checkNumberMargins(calibrationMatrix, marginMatrix))
     stop("Number of modalities must match between calibrationMatrix and marginMatrix to regroup calibration modalities.")
-
+  
   newCalibrationMatrix <- calibrationMatrix
   newMarginMatrix <- marginMatrix
-
+  
   ## Modification in calibrationMatrix
   newCalibrationMatrix[calibrationVariable] <- regroupUnContiguuousModalities(data.matrix(newCalibrationMatrix[calibrationVariable]), vecModalities, newModality)
-
+  
   ## Modification in marginMatrix
-  # Rank of modified modalities in calibrationMatrix
   calVarModalities <- unique(data.matrix(calibrationMatrix[calibrationVariable]))
-  modalitiesRanks <- rank(calVarModalities)
-  rankModalitiesDF <- as.data.frame(cbind(calVarModalities,modalitiesRanks))
-
-  vecRanks <- rankModalitiesDF[data.matrix(rankModalitiesDF[calibrationVariable]) %in% vecModalities,]["modalitiesRanks"][,1]
+  
+  if(newModality %in% calVarModalities) {
+    stop("New modality cannot be a modality that already exists in calibration matrix")
+  }
+  
+  orderedCalVarModalities <- calVarModalities[order(calVarModalities)]
+  indicesVecModalities <- which(orderedCalVarModalities %in% vecModalities)
+  indicesVecModalities <- indicesVecModalities+2 ## First two columns are name and nModalities
+  
   modifiedLine <- marginMatrix[marginMatrix[,1] == calibrationVariable,]
-
-  # Modify number of modalities in second column of marginMatrix
-  # If number of modalities == 1, stop
-  oldModalitiesNumber = as.numeric(modifiedLine[2])
-  modifiedLine[2] <- as.character(oldModalitiesNumber - length(vecModalities) + 1)
-
-  if( modifiedLine[2] == "1" ) {
-    stop("Margin matrix cannot reduce non-numeric margins to numeric margins")
-  }
-
-  regroupSum = sum(as.numeric(modifiedLine[vecRanks+2]))
-
-  # Delete regrouped modalities from line
-  deletedModalities <- vecRanks
-  modifiedLine[deletedModalities+2] <- NA
-  modifiedLine <- modifiedLine[!is.na(modifiedLine)]
-
+  sumRegrouped <- sum(as.numeric(modifiedLine[indicesVecModalities]))
+  modifiedLine <- modifiedLine[-indicesVecModalities]
+  
+  # Insert new margin (sum) to the right place
+  modifiedLine <- modifiedLine[modifiedLine != 0]
   newCalVarModalities <- unique(data.matrix(newCalibrationMatrix[calibrationVariable]))
-  newModalitiesRanks <- rank(newCalVarModalities)
-  rankNewModalitiesDF <- as.data.frame(cbind(newCalVarModalities,newModalitiesRanks))
-
-  insertPosition = rankNewModalitiesDF[rankNewModalitiesDF[,1] == newModality,]$newModalitiesRanks # Rank of new modality in calibrationMatrix modalities
-
-  # Insert regroupSum in modifiedLine
-  if(insertPosition+2 > length(modifiedLine)) {
-    modifiedLine[insertPosition+2] <- regroupSum
-  } else {
-    modifiedLine <- c(modifiedLine[1:(insertPosition+2-1)],regroupSum,modifiedLine[(insertPosition+2):length(modifiedLine)])
-  }
-
+  orderedNewCalVarModalities <- newCalVarModalities[order(newCalVarModalities)]
+  
+  insertPosition <- which(orderedNewCalVarModalities==newModality)
+  modifiedLine <- c(modifiedLine[1:(2+insertPosition-1)],sumRegrouped,
+                    modifiedLine[(2+insertPosition):length(modifiedLine)])
+  
+  newNModalities <- as.numeric(modifiedLine[2]) - length(vecModalities) + 1
+  modifiedLine[2] <- newNModalities
+  
+  
   # Add 0s to end line
+  modifiedLine <- modifiedLine[1:(as.numeric(modifiedLine[2])+2)]
   modifiedLine <- c(modifiedLine, rep("0.0000",ncol(marginMatrix) - length(modifiedLine)))
-
+  
   # Careful, sum of weights must be equal to 1 even after modalities have been regrouped
   sumMarginLine <- sum(as.numeric(modifiedLine[3:length(modifiedLine)]))
-
+  
   if( sumMarginLine != 1 ) {
-    #print(modifiedLine)
-    maxMarginValue <- max(as.numeric(modifiedLine[3:as.numeric(modifiedLine[2])+2]))
-    #print(maxMarginValue)
-    maxIndex <- which.max(as.numeric(modifiedLine[3:as.numeric(modifiedLine[2])+2]))
-    #print(maxIndex)
-    modifiedLine[maxIndex+2] <- maxMarginValue + 1 - sumMarginLine
-    #print(modifiedLine)
-  }
 
+    maxMarginValue <- max(as.numeric(modifiedLine[3:(as.numeric(modifiedLine[2])+2)]))
+    maxIndex <- which(as.numeric(modifiedLine[3:length(modifiedLine)]) == maxMarginValue)
+    modifiedLine[maxIndex+2] <- maxMarginValue + 1 - sumMarginLine
+  }
+  
   # Replace in marginMatrix
   newMarginMatrix[marginMatrix[,1] == calibrationVariable,] <- modifiedLine
-
+  
   # Check if last column of margin matrix is all 0s. If it is, drop last column
   # (means larger line has been reduced). Continue to do so until last colmun is not only 0s.
   while( sum(as.numeric(newMarginMatrix[,ncol(newMarginMatrix)])) == 0 ) {
     newMarginMatrix <- newMarginMatrix[, -ncol(newMarginMatrix)]
   }
-
+  
   eval.parent(substitute(calibrationMatrix <- newCalibrationMatrix))
   eval.parent(substitute(marginMatrix <- newMarginMatrix))
 }
@@ -577,6 +595,10 @@ regroupCalibrationModalities <- function(calibrationMatrix, marginMatrix, calibr
 #' @export
 addMargin <- function(marginMatrix, varName, vecTotals, adjustToOne=TRUE, thresholdAdjustToOne = 0.01) {
 
+  if(varName %in% marginMatrix[,1]) {
+    stop(paste(varName,"is already in margin matrix."))
+  }
+  
   newMarginMatrix <- marginMatrix
 
   # Length of vecTotals :
